% This file performs a specialized forward fit analysis on a specific
% spectrum. I really need to write a better way of doing this rather than
% hacking it together from the original analysis.
%--------------------------------------------------------------------------
%   Copyright (c) 2016 - 2018 Charles Swanson, Princeton Plasma Physics Lab
%   Copyright (c) 2019 Princeton Satellite Systems
%   All rights reserved.
%--------------------------------------------------------------------------

close all;clear;

controlString = {'File','16-SDD1-3divs.mca','Noise file','02-SDD1-CC-radialScan-3.5divs-200ns-noise.mca','Path','../spectra/','Channels','[40:1024]','Window model','NIST default thicknesses','Solver tolerance',1.00E-06,'Output directory','ForwardFitOutput','Comment','Line of sight: 5 cm upstream, 3.5 cm radial intercept. Assuming 8 cm radius','Plot','Yes','Save','Yes','Save plot','Yes'};

localPath = GetKeyWordValue('Path',controlString);
MCA = ParseMCA(strcat(localPath,GetKeyWordValue('File',controlString)));
MCA = MCA.SetECal(MCA);
fprintf('%s \n',MCA.fileName);

noiseFile = GetKeyWordValue('Noise file',controlString);
MCANoise = ParseMCA(strcat(localPath,noiseFile));
MCANoise = MCANoise.SetECal(MCANoise);
sigmaIntrinsic = MCANoise.ECal.slope*MCAIntrinsicNoise(MCANoise);

target_pressure = LoadMetadata(MCA,'Pressure (mTorr)');
Z = LoadMetadata(MCA,'Zeff');
atoms_in_molecule = LoadMetadata(MCA,'Atoms per molecule');
dutyFactor = LoadMetadata(MCA,'Duty factor');
nMolecules = target_pressure*3.2e13;%in /cc

mount = LoadMount(LoadMetadata(MCA,'Mount'));

channels = GetKeyWordValue('Channels',controlString);
% Ensure compatibility with both Octave and MATLAB
if sum(version=='R')
  channels = eval(channels);
end
solver_tolerance = GetKeyWordValue('Solver tolerance',controlString);
comment = GetKeyWordValue('Comment',controlString);

if sum(MCA.counts(channels)) == 0
  XRaySpectrum(controlString);
  return
end

%% Set up the detector response
bremssMatrix = BremssModel(MCA.ECal.Ex,MCA.ECal.Ex',nMolecules,MCA.ECal.slope,atoms_in_molecule,Z);
resolutionMatrix = GaussianResponse(MCA.ECal.Ex,MCA.ECal.Ex',SigmaFano(MCA.ECal.Ex,sigmaIntrinsic),MCA.ECal.slope);
if strcmp(GetKeyWordValue('Window model',controlString),'NIST default thicknesses')
  windowMatrix = diag(C1NISTEfficiency(MCA.ECal.Ex));
end
if strcmp(GetKeyWordValue('Window model',controlString),'Amptek table')
  windowMatrix = diag(C1TablesEfficiency(MCA.ECal.Ex));
end

%% Set up the transformation and cost functions
fileList = readcell('./ForwardFitOutput/ForwardFitResults.csv'); 
controlString2 = fileList(strcmp(MCA.fileName,fileList(:,2)),3:end);
T = GetKeyWordValue('Temperature (eV)',controlString2)
n = GetKeyWordValue('Density (/cc)',controlString2)

f=[n T];
f_maxwellian=@(Ee,T) 2*sqrt(Ee/pi).*T.^(-3/2).*exp(-Ee./T);%units of /eV
L_eedf=@(E,f) bsxfun(@times,f(:,1,:),f_maxwellian(E,f(:,2,:)));
L_elements = resolutionMatrix*windowMatrix*bremssMatrix;
L = @(f) MCA.ECal.slope*MCA.timeAccum*dutyFactor*mount.V*sum(L_elements.*L_eedf(MCA.ECal.Ex',f),2); % Can't be a standard matrix multiplication because f has dimension 3

%% Solver
cost=@(LOfF) PoissonLogLikelihood(MCA.counts(channels),LOfF(channels,:,:));
CostQN = @(f) cost(L(f));

fprintf('Fit density is %.4g /cc. \n',f(1));
fprintf('Fit temperature is %.4g eV. \n',f(2));

%% Plot the spectrum and fit
fig = figure('Position', [100, 100, 600, 400]);
figFont = 12;figLineWidth = 1;
figtitle={MCA.dateTime,MCA.fileName,comment};

mPlot = MCA.counts/MCA.ECal.slope/MCA.timeAccum/dutyFactor/mount.V;
mErrs = sqrt(MCA.counts)/MCA.ECal.slope/MCA.timeAccum/dutyFactor/mount.V;
LPlot = @(f) L(f)/MCA.ECal.slope/MCA.timeAccum/dutyFactor/mount.V;

plotRange = (1:1024);
plotRange = plotRange(MCA.ECal.Ex>270);
LVec = LPlot(f);

hold on;
errorbar(MCA.ECal.Ex(plotRange),mPlot(plotRange:end),mErrs(plotRange),'.','LineWidth',figLineWidth);
plot(MCA.ECal.Ex(plotRange),LVec(plotRange),'LineWidth',figLineWidth);
set(gca,'yscale','log');
legend('Measured',strcat('n_e=',num2str(f(1),'%10.2e'),'/cc, T_e=',num2str(f(2),3),'eV'));
axis([0 1.1*max(MCA.ECal.Ex(MCA.counts>0)) min(mPlot(mPlot~=0))/2 1.1*max(mPlot)]);
xlabel('Energy (eV)','FontSize',figFont);
ylabel('Volumetric spectral X-Ray Rate (/cc/eV/s)','FontSize',figFont);
set(gca,'FontSize',figFont,'LineWidth',figLineWidth);
grid on;grid minor;
title(figtitle,'FontSize',figFont);
axis([0 1000 1e2 1e6])

saveas(gcf,'SpecialSpectrum16.pdf');
